/* This file contains carbon dynamics related functions used by the kiwifruit shoot L-system model. */
/* If you make any changes, don't forget to save it before running the model. */

#ifndef _FUNCTIONS_H
#define _FUNCTIONS_H

/* -------------------------------------------------------------------------- */
// common sink/source functions

float LogisticGrowthRate (float dev_age)
// defines a logistic growth rate based on developmental age
{
  return (1.0 / (1.0 + exp(dev_age))); 
}

float SinkUnloading (float c, float q)
// defines carbon unloading
// c is carbon concentration (g C / cm^3)
// q is priority parameter (g C / cm^3)
{
  return (c / (q + c));
}

float dSinkUnloading_dc (float c, float q)
// defines the derivative with respect to 'c' of carbon unloading function
// c is carbon concentration (g C / cm^3)
// q is priority parameter (g C / cm^3)
{
  return (q / ((q + c)*(q + c)));
}

float SourceLoading (float c, float q)
// c is carbon concentration (g C / cm^3)
// q is loading rate parameter
{
  // source loading = -1 + (c / (q + c))
  return (-1.0 + c / (q + c));
}

float dSourceLoading_dc (float c, float q)
// defines the derivative with respect to 'c' of carbon loading function
{
  return (q / ((q + c) * (q + c)));
}


/* -------------------------------------------------------------------------- */
// apex specific functions

float ApexRateofAppearance (void)
{
  // phyllochron should be a function of temperature
  return (1.0 / PHYLLOCHRON);
}

int ApexGrowthCessation (float c, float vigour, float node)
// where 'c' is the carbon concentration outside the apex
// and 'vigour' is the shoot's growth potential
{
  // if concentration is less than threshold, check for growth cessation
  if (c <= CARBON_THRESHOLD)
  {
    Printf ("APEX ABORT\n");
    return (1);
  }

  return (0);
}


/* -------------------------------------------------------------------------- */
// leaf specific functions

float LeafSpecificGrowthRate (void)
{
  return (1.0 / TAU_LEAF);
}

float LeafLowerAsymptote (float node, float order)
// defines the lower asymptote of the Boltzmann function (like logistic but with extra parameter)
{
  static float B[17] =
    {-6.515,-12.772,-18.733,-24.353,-29.572,-34.322,
     -38.532,-27.439,-19.677,-13.848,-9.586,-6.539,
     -4.402,-2.928,-1.927,-1.264,-0.865};

  const float scale_factor = 0.45;

  if (node <= 17 && order <= 1)
    return (B[(int)node-1]*LEAF_BIO2CARB*LEAF_DENSITY*scale_factor);
  else
    return (B[16]*LEAF_BIO2CARB*LEAF_DENSITY*scale_factor);
}

float LeafPotentialGrowthRate (float carbon, float dev_age, float node, float order)
// parameter 'carbon' is leaf structural carbon
{
  return ((carbon - LeafLowerAsymptote(node,order)) * LogisticGrowthRate(dev_age) / TAU_LEAF);
}

float LeafMaintenance (float carbon)
// parameter 'carbon' is leaf structural carbon
{
  const float temp_response = log(2.0) / 10.0;
  const float temp_avg = 0.5 * (12.0 + 17.0);

  return(carbon * MTNCE_INDE * exp(temp_response * (temp_avg-20.0)));
}

float LeafCarbonOutput (float carbon)
// 'carbon' is carbon accumulated from photosynthesis
{
  return (LEAF_LOADING * carbon);
}

float LeafCarbonFromPhoto (float carbon, float structural_carbon, float acquired_carbon, float area)
// returns the amount of carbon acquired in leaf from photosynthesis
// carbon gives what is already there (stored as starch?)
// structural_carbon is size of the leaf (sink) in g C
// acquired_carbon is g C / (cm^2 day), so area is also the size of the leaf
{
  if (acquired_carbon * area * DT + carbon > structural_carbon)
    return (structural_carbon - carbon);
  else
    return (acquired_carbon * area);
}

float Carb2Area (float q)
{
  // return area of leaf given amount of carbon
  return (q / (LEAF_BIO2CARB*LEAF_DENSITY));
}

float LeafInitialDevAge (float node, float order)
// defines the initial developmental age of a leaf by node number
{
  static float alpha[17] =
    {-1.964,-1.983,-2.003,-2.023,-2.043,-2.063,-2.083,
     -2.531,-2.951,-3.371,-3.790,-4.210,-4.630,-5.049,
     -5.469,-5.889,-6.233};

  if (order == 1)
  {
    if (node <= 17)
      return (alpha[(int)node-1]);
    else 
      return (alpha[16]+(node-17)*0.00461);
  }
  else // all leaves are preformed
    return (alpha[16]+node*0.00461);
}


/* -------------------------------------------------------------------------- */
// internode specific functions

float IndeSpecificGrowthRate (void)
{
  return (1.0 / TAU_INDE);
}

float IndePrimaryGrowthRate (float dev_age, float length)
{
  return (LogisticGrowthRate(dev_age) * length / TAU_INDE);
}

float IndeSecondaryGrowthRate (float dev_age, float radius)
{
  return ((SEC_GROWTH_RATE +
           SEC_LOGISTIC_RATE * LogisticGrowthRate(dev_age) * radius) / TAU_INDE);
}

float IndeMaintenance (float carbon, float order)
// parameter 'carbon' is structural carbon
{
  const float temp_response = log(2.0) / 10.0;
  const float temp_avg = 0.5 * (12.0 + 17.0);

  if (order == CANE_ORDER)
    return(carbon * MTNCE_CANE * exp(temp_response * (temp_avg-20.0)));
  else
    return(carbon * MTNCE_INDE * exp(temp_response * (temp_avg-20.0)));
}

float IndeReserveSynthesis (float carbon, float stored_carbon, float syn_rate)
// where 'carbon' is structural carbon
// and 'stored_carbon' is the carbon in reserves
{
  if (carbon*INDE_RESERVES < stored_carbon)
    Printf ("Warning: internode stored carbon is > maximum possible!\n");

  return (syn_rate * (carbon*INDE_RESERVES - stored_carbon));
}

float IndeReserveHydrolysis (float stored_carbon, float hyd_rate)
{
  return (stored_carbon*hyd_rate);
}

float IndeFlowResistance (float length, float radius, float resistance)
{
  return (resistance * length / (2.0 * M_PI * radius));
}

float IndeInitialLength (float node, float order)
{
  const float scale_factor = 0.45;

  // does initial internode length depend on shoot order?
  if (node <= 21)
    return (0.24+0.105*(node-1)) * scale_factor;
  else
    return (115-0.72*node)/45.0 * scale_factor;
}

float IndeInitialDevAge (void)
// defines the initial developmental age of an internode
{
  return (-INDE_TIMING_EXPANSION/TAU_INDE);
}


/* -------------------------------------------------------------------------- */
// flower/fruit specific functions

int FlowerDevelopment (float node)
// returns 1 (true) if flower should develop at this node number, else 0 (false)
{
  // probabilities taken from Walton and Fowke(1993):
  // Fig. 1 (top), shaded bars, nodes 5 to 12, as we assume only 8 floral nodes
  float prob_floral_develop[8] = {0.05, 0.2, 0.4, 0.7, 0.8, 0.65, 0.5, 0.22};
  int nodei = (int) (node - 1.0);

  if (node <= 8.0)
  {
    if (ran(1) < prob_floral_develop[nodei])
      return (1);
  }

  return (0);
}

float FlowerGrowthRate (float final_carbon)
{
  return (final_carbon / MAX_FLOWER_AGE);
}

float FruitGrowthRate (float carbon)
// growth rate function from Gandar, Hall and de Silva (19??).
{
  float k1 = 0.020749;
  float k2 = 22.213;
  float k3 = 0.63708;

  // the parameter values above assume the final carbon is 1.0
  // so I need to normalize by the actual maximum.
  float norm = carbon / MAX_FRUIT_CARBON;

  return MAX_FRUIT_CARBON * (k1 * norm * (1.0 - norm) * (1.0 + k2*(norm-k3)*(norm-k3)));
  //return (k1 * carbon * (1.0 - carbon/MAX_FRUIT_CARBON) * (1.0 + k2*(carbon-k3)*(carbon-k3)));
}

float FruitMaintenance (float carbon)
{
  const float temp_response = log(2.0) / 10.0;
  const float temp_avg = 0.5 * (12.0 + 17.0);

  return(carbon * MTNCE_FRUIT * exp(temp_response * (temp_avg-20.0)));
}


/* -------------------------------------------------------------------------- */
// cane specific functions

float CaneIndeInitialReserves (float structural_carbon)
{
  // INDE_RESERVES=0.0592 is from 20/9 gDW / gC * 4/9 gC / gStarch * 3/50 gStarch/ gDW
  // the last constant is from Greaves et al (1999)

  return (structural_carbon * INDE_RESERVES);
}


/* -------------------------------------------------------------------------- */

#endif
